/*
** Job Arranger for ZABBIX
** Copyright (C) 2012 FitechForce, Inc. All Rights Reserved.
** Copyright (C) 2013 Daiwa Institute of Research Business Innovation Ltd. All Rights Reserved.
** Copyright (C) 2021 Daiwa Institute of Research Ltd. All Rights Reserved.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/
#include "common.h"
#include "log.h"
#include "db.h"

#include "jacheckjob.h"
#include "jacommon.h"
#include "jastatus.h"
#include "jalog.h"
#include "jajoblog.h"
#include "jahost.h"
#include "jastr.h"
#include "jaself.h"
#include "jarunsessioncheck.h"
#include "jarunstatuscheck.h"
#include <time.h>
#include <stdio.h>

extern unsigned char process_type;
extern int process_num;
static int timeout_cnt = 0;
static int u_time = 0;
static time_t start_time, current_time;
static int TRUE = 1;
static int FALSE = -1;

/******************************************************************************
 *                                                                            *
 * Function: check_size_of_int_list()                                         *
 *                                                                            *
 * Purpose: check and increase the size of the int list                       *
 *                                                                            *
 * Parameters: total count of the list,                                       *
 *             size of the original list,                                     *
 *             original list                                                  *
 *                                                                            *
 * Last Updated by: Nyan Linn Soe (8.31.2024)                                 *
 *                                                                            *
 ******************************************************************************/
void check_size_of_int_list(int count, int *size_of_list,  int **list){
    if (count > *size_of_list) {
        *size_of_list += 512;
        int* resized_list = (int*)realloc(*list, *size_of_list * sizeof(int));
        if (resized_list != NULL) {
            *list = resized_list;
        }else {
            zabbix_log(LOG_LEVEL_ERR, "In check_size_of_int_list(), memory reallocation failed for resize list");
        }
    }
}

/******************************************************************************
 *                                                                            *
 * Function: check_size_of_char_list()                                        *
 *                                                                            *
 * Purpose: check and increase the size of the 2D character list (array of    *
 *          strings)                                                          *
 *                                                                            *
 * Parameters: total count of the list,                                       *
 *             size of the original list,                                     *
 *             original list                                                  *
 *                                                                            *
 * Last Updated by: Nyan Linn Soe (8.31.2024)                                 *
 *                                                                            *
 ******************************************************************************/
void check_size_of_char_list(int count, int *size_of_list, char (**list)[256]) {
    if (count > *size_of_list) {
        *size_of_list += 10;  // Increase the size by 10 (or any other step you prefer)
        char (*resized_list)[256] = (char(*)[256])realloc(*list, *size_of_list * sizeof(*resized_list));
        
        if (resized_list != NULL) {
            *list = resized_list;
        } else {
            zabbix_log(LOG_LEVEL_ERR, "In check_size_of_char_list(), memory reallocation failed for resize list");
        }
    }
}


/******************************************************************************
 *                                                                            *
 * Function: getRebootRunning()                                               *
 *                                                                            *
 * Purpose: get running reboot icon from the host                             *
 *                                                                            *
 * Parameters: sql_flag to varify if hostname is used with variable,          *
 *             target hostname,                                               *
 *             list of jobs which is going to be inserted                     *
 *             count of jobs                                                  *
 *             list of reboot job which is going to be inserted               *
 *                                                                            *
 * Last Updated by: Nyan Linn Soe (8.31.2024)                                 *
 *                                                                            *
 ******************************************************************************/
int* getRebootRunning(int sql_flag, char* hostname, int* joblist, int count_job,int* reboot_job_list){
    
    const char *__function_name = "check_reboot_process";
    DB_RESULT result;
    DB_ROW row;
    int size_of_joblist=512;
    int size_of_reboot_joblist=512;
    int count_reboot_job = 0; 
    if (sql_flag == 0) {
        result = DBselect("select b.inner_job_id"
            " from ( select a.inner_job_id,a.inner_jobnet_id from ja_run_job_table as a"
            " where a.job_type = %d and a.status = %d ) as b,"
            " ja_run_icon_reboot_table as c"
            " where c.host_flag = 0 and b.inner_job_id = c.inner_job_id and c.host_name = '%s'"
            " order by c.host_name asc", JA_JOB_TYPE_REBOOT, JA_JOB_STATUS_RUN,
            hostname);
    } else {
        result = DBselect("select e.inner_job_id"
            " from"
            " ( select g.inner_job_id, max(g.seq_no) as max_seq_no"
            " from ja_run_value_before_table g,"
            " ( select c.host_name, b.inner_job_id"
            " from ( select a.inner_job_id from ja_run_job_table as a"
            " where a.job_type = %d and a.status = %d ) as b,"
            " ja_run_icon_reboot_table as c"
            " where c.host_flag = 1 and b.inner_job_id = c.inner_job_id) as d"
            " where g.inner_job_id = d.inner_job_id and g.value_name = d.host_name"
            " group by g.inner_job_id ) as e,"
            " ja_run_value_before_table f"
            " where e.max_seq_no = f.seq_no and f.before_value = '%s'"
            " order by f.before_value asc",
            JA_JOB_TYPE_REBOOT, JA_JOB_STATUS_RUN, hostname);
    }
    
    while (NULL != (row = DBfetch(result))) {
        check_size_of_int_list(count_job,&size_of_joblist,&joblist);
        joblist[count_job] = atoi(row[0]);
        check_size_of_int_list(count_reboot_job,&size_of_reboot_joblist,&reboot_job_list);
        reboot_job_list[count_reboot_job] = atoi(row[0]);
        count_reboot_job++;
        count_job++;
    }
    reboot_job_list[count_reboot_job] = -1;

    DBfree_result(result);
    return count_reboot_job;
} 

/******************************************************************************
 *                                                                            *
 * Function: check_reboot_process()                                           *
 *                                                                            *
 * Purpose: check if there is still running reboot job left for               *
 *          different host                                                    *
 *                                                                            *
 * Parameters: count of host which has been checked,                          *
 *             list of host which has been checked,                           *
 *             sql_flag to varify if hostname is used with variable           *
 *                                                                            *
 * Last Updated by: Nyan Linn Soe (8.31.2024)                                 *
 *                                                                            *
 ******************************************************************************/
void check_reboot_process(int reboot_host_count, char (*reboot_host_list)[256], int sql_flag){
    const char *__function_name = "check_reboot_process";

    int host_match;
    int index;
    int count_reboot_job = 0;
    int str_cmp_result = 0;
    int size = 512;
    int* joblist = (int*)malloc(size * sizeof(int));
    int* reboot_job_list = (int*)malloc(size * sizeof(int));

    char hostname[256];

    DB_ROW row1;
    DB_ROW row2;
    DB_RESULT result1;
    DB_RESULT result2;

    hostname[0] = 0x00;
    reboot_job_list[0] = -1;
    result1 = DBselect("select count(*) from ja_host_lock_table"
                      " WHERE lock_host_name != 'HOST_LOCK_RECORD'");

	row1 = DBfetch(result1);
    index = 0;
    if (row1 != NULL) {
        if( atoi(row1[0]) > reboot_host_count ){
            result2 = DBselect("SELECT lock_host_name FROM ja_host_lock_table"
                       " WHERE lock_host_name != 'HOST_LOCK_RECORD'"
                       " order by lock_host_name asc");
            while ((row2 = DBfetch(result2)) != NULL){
                zbx_strlcpy(hostname, row2[0], sizeof(hostname));
                host_match = FALSE;
                while(index < reboot_host_count){
                    str_cmp_result =  strcmp(hostname, reboot_host_list[index]);
                    if( str_cmp_result == 0  ){
                        host_match = TRUE;
                        index++;
                        break;
                    }
                    else if(str_cmp_result > 0)  index++;
                    else   break;
                }
                ja_sleep_loop(1);
                if(host_match != TRUE){
                    count_reboot_job = getRebootRunning(sql_flag, hostname, joblist, count_reboot_job, reboot_job_list);
                    if( count_reboot_job > 0 ){
                        jarun_status_check(hostname, joblist, count_reboot_job, reboot_job_list);
                        ja_log("JACHECKJOB000002", 0, NULL, 0, __function_name, 0, count_reboot_job, hostname, sql_flag);
                    }
                }
            }
        DBfree_result(result2);
        }
    }
    DBfree_result(result1);
}

/******************************************************************************
 *                                                                            *
 * Function: check_session_process()                                          *
 *                                                                            *
 * Purpose: to check running session on server side                           *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 * Last Updated by: Htoo Zaw Aung                                             *
 *                                                                            *
 ******************************************************************************/
void check_session_process(){
    const char *__function_name = "check_session_process";
    DB_RESULT session_result;
    DB_ROW session_row;
    
    int count_pid = 0;
    int size = 512;

    pid_t* pid_list = (pid_t*)malloc(size * sizeof(int));

    zabbix_log(LOG_LEVEL_DEBUG,"%s() has started.", __function_name);

    session_result = 
        DBselect(" select pid,start_time,jst.inner_job_id,session_id,jst.inner_jobnet_main_id "
                 " from ja_session_table jst, ja_run_job_table jrjt"
                 " where jst.inner_job_id = jrjt.inner_job_id and jrjt.status = %d;",
                 JA_JOB_STATUS_RUN);
            
    while(NULL != (session_row = DBfetch(session_result))) {
        if (count_pid > size) { 
            size += 512;
            pid_t* resized_pidlist = (pid_t*)realloc(pid_list, size * sizeof(int));
            pid_list = resized_pidlist;
        }
        pid_t tmp_pid = atoi(session_row[0]);
        time_t start_time, current_time;
        zbx_uint64_t run_inner_job_id;
        char run_session_id[JA_MAX_STRING_LEN];
        zbx_uint64_t run_inner_jobnet_main_id;

        ZBX_STR2UINT64(run_inner_job_id, session_row[2]);

        if(tmp_pid==0){
            zabbix_log(LOG_LEVEL_DEBUG,"In %s(), agentless is running while session process pid is zero.inner_job_id:"ZBX_FS_UI64,__function_name,run_inner_job_id);
            if(ja_is_session_dbc_avaliable() == SUCCEED){
                start_time = ja_str2timestamp(session_row[1]);
                current_time = time(NULL);
                zabbix_log(LOG_LEVEL_DEBUG,"In %s(), agentless start time:%d.inner_job_id:"ZBX_FS_UI64,__function_name,start_time,run_inner_job_id);
                if(current_time > start_time + 3600){
                    zbx_strlcpy(run_session_id,session_row[3],sizeof(run_session_id));
                    ZBX_STR2UINT64(run_inner_jobnet_main_id, session_row[4]);
                    //Add ja_log for message :-> new logmessage.
                    ja_log("JARUNICONLESS200011",0,NULL,run_inner_job_id,__function_name,run_inner_job_id,run_session_id,tmp_pid);
                    if(ja_set_runerr(run_inner_job_id, 2) == SUCCEED) {
                        DBexecute(" delete from ja_session_table" 
                                  " where session_id = '%s' and inner_jobnet_main_id ="ZBX_FS_UI64,
                                  run_session_id,run_inner_jobnet_main_id);
                    } 
                }
            }
            continue;
        }
        pid_list[count_pid] = tmp_pid;
        count_pid++;
    }
    jarun_session_check(pid_list, count_pid);
    DBfree_result(session_result);
    free(pid_list);
    zabbix_log(LOG_LEVEL_DEBUG,"%s() has finished.", __function_name);
}             

/******************************************************************************
 *                                                                            *
 * Function: check_job_process()                                              *
 *                                                                            *
 * Purpose: to check the running process on agent side for                    *
 *          1) job icon                                                       *
 *          2) fwait icon                                                     *
 *          3) reboot icon                                                    *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 * Last Updated by: Nyan Linn Soe (8.31.2024)                                 *
 *                                                                            *
 ******************************************************************************/
void check_job_process(){
    const char *__function_name = "check_job_process";
    char hostname[256];
    char (*reboot_host_list)[256];
    
    int ret;
    int sql_flag = 0;
    int count_job = 0;
    int count_reboot_job = 0;
    int reboot_host_count= 0;
    int size = 512;
    int size_of_hostname = 256;
    int size_of_joblist;
    int size_of_reboot_host_list;
    
    int* joblist;
    int* reboot_job_list;

    DB_ROW row;
    DB_RESULT result;
    
    
    zabbix_log(LOG_LEVEL_DEBUG,"%s() has started.", __function_name);

    while(TRUE){
        //memory allocation for list which can contain up to 512 "job id"
        joblist = (int*)malloc(size * sizeof(int));
        reboot_job_list = (int*)malloc(size * sizeof(int));
        
        //memory allocation for list which can contain up to 10 "hostname[256]""
        reboot_host_list = (char(*)[256])malloc(10 * sizeof(*reboot_host_list));

        if(joblist == NULL || reboot_job_list == NULL || reboot_host_list == NULL){
            if(joblist != NULL) free(joblist);
            if(reboot_job_list != NULL) free(reboot_job_list);
            if(reboot_host_list != NULL) free(reboot_host_list);
            zabbix_log(LOG_LEVEL_ERR,"In %s(), memory allocation failed!! [stderror:%s]", __function_name, zbx_strerror(errno));
            break;
        }

        hostname[0] = 0x00;
        reboot_job_list[0] = -1;
        count_job = 0;
        count_reboot_job = 0;
        size_of_joblist=512;
        size_of_reboot_host_list=10;

        //if hostname was used with variable icon => sql_flag = 1
        if(sql_flag == 0) {
            result = DBselect("select c.host_name, b.inner_job_id, b.inner_jobnet_id"
                                " from ( select a.inner_job_id,a.inner_jobnet_id from ja_run_job_table as a"
                                " where a.job_type in(%d,%d) and a.status in(%d,%d)) as b," 
                                " (SELECT inner_job_id, host_name FROM ja_run_icon_job_table WHERE host_flag = 0"
                                " UNION ALL"
                                " SELECT inner_job_id, host_name FROM ja_run_icon_fwait_table WHERE host_flag = 0)"
                                " AS c"
                                " where b.inner_job_id = c.inner_job_id"
                                " order by c.host_name asc",
                                JA_JOB_TYPE_JOB, JA_JOB_TYPE_FWAIT, JA_JOB_STATUS_ABORT, JA_JOB_STATUS_RUN);
        }else{
            result = DBselect("select  f.before_value as host_name, e.inner_job_id, f.inner_jobnet_id, f.seq_no"
                                " from"
                                " ( select g.inner_job_id, max(g.seq_no) as max_seq_no"
                                " from ja_run_value_before_table g,"
                                " ( select c.host_name, b.inner_job_id"
                                " from ( select a.inner_job_id from ja_run_job_table as a"
                                " where a.job_type in(%d,%d) and a.status in(%d,%d) ) as b,"
                                " (SELECT inner_job_id, host_name FROM ja_run_icon_job_table WHERE host_flag = 1"
                                " UNION ALL"
                                " SELECT inner_job_id, host_name FROM ja_run_icon_fwait_table WHERE host_flag = 1)"
                                " AS c"
                                " where b.inner_job_id = c.inner_job_id) as d"
                                " where g.inner_job_id = d.inner_job_id and g.value_name = d.host_name"
                                " group by g.inner_job_id ) as e,"
                                " ja_run_value_before_table f"
                                " where e.max_seq_no = f.seq_no"
                                " order by f.before_value asc",
                                JA_JOB_TYPE_JOB, JA_JOB_TYPE_FWAIT, JA_JOB_STATUS_ABORT,JA_JOB_STATUS_RUN);
        }
        
        while (NULL != (row = DBfetch(result))){
            if (hostname[0] == 0x00){
                zbx_strlcpy(hostname, row[0], sizeof(hostname));
            }
            if(!count_job){
                zabbix_log(LOG_LEVEL_DEBUG,
                    "In %s(), checking status for hostname: %s with sql_flag: %d has started.",
                    __function_name, hostname, sql_flag);
            }
             
            if (strcmp(hostname, row[0]) != 0 ) {
                //different hostname is found, preparing for sending request to host
                if (ja_host_lockinfo(hostname) == SUCCEED) {
                    check_size_of_int_list(reboot_host_count, &size_of_reboot_host_list, &reboot_host_list);
                    zbx_strlcpy(reboot_host_list[reboot_host_count], hostname, 256);
                    reboot_host_count++;
                    count_reboot_job = getRebootRunning(sql_flag, hostname, joblist, count_job,reboot_job_list);
                }
                jarun_status_check(hostname, joblist, count_job,reboot_job_list);
                ja_log("JACHECKJOB000002", 0, NULL, 0, __function_name, count_job, count_reboot_job, hostname, sql_flag);

                //prepare for next hostname
                free(joblist);
                free(reboot_job_list);
                zbx_strlcpy(hostname, row[0], sizeof(hostname));
                count_job = 0;
                size_of_joblist = 512;
                size_of_reboot_host_list = 512;
                joblist = (int*)malloc(size * sizeof(int));
                reboot_job_list = (int*)malloc(size * sizeof(int));
            }
            
            check_size_of_int_list(count_job, &size_of_joblist, &joblist);
            joblist[count_job] = atoi(row[1]);
            count_job++;
        }

        // this is for last/single hostname 
        if(count_job > 0){
            if (ja_host_lockinfo(hostname) == SUCCEED) {
                check_size_of_char_list(reboot_host_count, &size_of_reboot_host_list, &reboot_host_list);
                zbx_strlcpy(reboot_host_list[reboot_host_count], hostname, size_of_hostname);
                reboot_host_count++;
                count_reboot_job = getRebootRunning(sql_flag, hostname, joblist, count_job,reboot_job_list);
            }
            jarun_status_check(hostname, joblist, count_job, reboot_job_list);
            ja_log("JACHECKJOB000002", 0, NULL, 0, __function_name, count_job, count_reboot_job, hostname, sql_flag);
        }
        free(joblist);
        free(reboot_job_list);
        check_reboot_process(reboot_host_count, reboot_host_list,sql_flag);
        free(reboot_host_list);
        if(sql_flag == 0) sql_flag = 1;
        else break;
    }
    DBfree_result(result);
    zabbix_log(LOG_LEVEL_DEBUG,"%s() has finished.", __function_name);
}

/******************************************************************************
 *                                                                            *
 * Function: main_jacheckjob_loop()                                           *
 *                                                                            *
 * Purpose: to check the status of job                                        *
 *                                                                            *
 * Last Updated by: Nyan Linn Soe (8.31.2024)                                 *
 ******************************************************************************/
void main_jacheckjob_loop() {
    const char *__function_name = "main_jacheckjob_loop";
	int jacheckjob_timeout_cnt = 30;
    int loop_cnt = 0;

    zabbix_log(LOG_LEVEL_DEBUG, "In %s() process_type:'%s' process_num:%d",
        __function_name, ja_get_process_type_string(process_type), process_num);

    zbx_setproctitle("%s [connecting to the database]", 
        ja_get_process_type_string(process_type));
    u_time = zbx_time();
    ja_alarm_watcher("ja_check_job");
    DBconnect(ZBX_DB_CONNECT_NORMAL);
    start_time = time(NULL);
    for (;;) {
        ja_alarm_timeout(CONFIG_CHECKJOB_TIMEOUT);
        zbx_setproctitle("%s [processing data]", ja_get_process_type_string(process_type));

        check_job_process();
        check_session_process();

        ja_alarm_timeout(0);

        current_time = time(NULL);
        if (current_time - start_time < CONFIG_JACHECKJOB_INTERVAL){
            zabbix_log(LOG_LEVEL_DEBUG,"In %s(), sleeping for %d seconds", 
                __function_name, CONFIG_JACHECKJOB_INTERVAL - (current_time - start_time));
            ja_sleep_loop(CONFIG_JACHECKJOB_INTERVAL - (current_time - start_time));
        }
        start_time = time(NULL);
        if (timeout_cnt >= jacheckjob_timeout_cnt)
            timeout_cnt = 0;
        else
            timeout_cnt += CONFIG_JACHECKJOB_INTERVAL;

    }
    
}